@@ -1,5 +1,5 @@
 #
-# $Id: IMDB.pm,v 1.46 2004/05/15 02:39:03 jveldhuis Exp $
+# $Id: IMDB.pm,v 1.53 2007/01/12 20:08:47 jveldhuis Exp $
 #
 # The IMDB file contains two packages:
 # 1. XMLTV::IMDB::Cruncher package which parses and manages IMDB "lists" files
@@ -33,7 +33,7 @@ use strict;
 
 package XMLTV::IMDB;
 
-our $VERSION = '0.5';
+our $VERSION = '0.6';
 
 sub new
 {
@@ -174,7 +174,7 @@ sub basicVerificationOfIndexes($)
 
     # check that the imdbdir is invalid and up and running
     my $title="Army of Darkness";
-    my $year=1993;
+    my $year=1992;
 
     $self->openMovieIndex() || return("basic verification of indexes failed\n".
 				      "database index isn't readable");
@@ -236,10 +236,10 @@ sub basicVerificationOfIndexes($)
 	return("basic verification of indexes failed\n".
 	       "movie details didn't show Bruce Campbell as the main actor in movie \"$title, $year\" (id=$id)\n");
     }
-    if ( $res->{genres}[0] ne "Adventure" ||
-	 $res->{genres}[1] ne "Fantasy" ||
-	 $res->{genres}[2] ne "Action" ||
-	 $res->{genres}[3] ne "Comedy" ||
+    if ( $res->{genres}[0] ne "Action" ||
+	 $res->{genres}[1] ne "Adventure" ||
+	 $res->{genres}[2] ne "Comedy" ||
+	 $res->{genres}[3] ne "Fantasy" ||
 	 $res->{genres}[4] ne "Horror" ) {
 	$self->closeMovieIndex();
 	return("basic verification of indexes failed\n".
@@ -331,7 +331,7 @@ sub getMovieMatches($$$)
     my $year=shift;
 
     # Articles are put at the end of a title ( in all languages )
-    #$match=~s/^(The|A|Une|Les|L\'|Le|La|El|Das)\s+(.*)$/$2, $1/og;
+    #$match=~s/^(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)\s+(.*)$/$2, $1/og;
     
     my $match="$title";
     if ( defined($year) ) {
@@ -365,7 +365,7 @@ sub getMovieMatches($$$)
 
 	if ( $arr[0] eq $match ) {
 	    # return title and id
-	    #$arr[1]=~s/(.*),\s*(The|A|Une|Les|L\'|Le|La|El|Das)$/$2 $1/og;
+	    #$arr[1]=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 		    
 	    #$arr[0]=~s/%(?:([0-9a-fA-F]{2})|u([0-9a-fA-F]{4}))/defined($1)? chr hex($1) : utf8_chr(hex($2))/oge;
 	    #$self->debug("exact:$arr[1] ($arr[2]) qualifier=$arr[3] id=$arr[4]");
@@ -377,7 +377,7 @@ sub getMovieMatches($$$)
 	    else {
 		die "unable to decode year from title key \"$title\", report to xmltv-devel\@lists.sf.net";
 	    }
-	    $title=~s/(.*),\s*(The|A|Une|Les|L\'|Le|La|El|Das)$/$2 $1/og;
+	    $title=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 	    $self->debug("exact:$title ($arr[2]) qualifier=$arr[3] id=$arr[4]");
 	    push(@{$results->{exactMatch}}, {'key'=> $arr[1],
 					     'title'=>$title,
@@ -389,7 +389,7 @@ sub getMovieMatches($$$)
 	    # decode
 	    #s/%(?:([0-9a-fA-F]{2})|u([0-9a-fA-F]{4}))/defined($1)? chr hex($1) : utf8_chr(hex($2))/oge;
 	    # return title
-	    #$arr[1]=~s/(.*),\s*(The|A|Une|Les|L\'|Le|La|El|Das)$/$2 $1/og;
+	    #$arr[1]=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 	    #$arr[0]=~s/%(?:([0-9a-fA-F]{2})|u([0-9a-fA-F]{4}))/defined($1)? chr hex($1) : utf8_chr(hex($2))/oge;
 	    #$self->debug("close:$arr[1] ($arr[2]) qualifier=$arr[3] id=$arr[4]");
 	    my $title=$arr[1];
@@ -406,7 +406,7 @@ sub getMovieMatches($$$)
 	    else {
 		die "unable to decode year from title key \"$title\", report to xmltv-devel\@lists.sf.net";
 	    }
-	    $title=~s/(.*),\s*(The|A|Une|Les|L\'|Le|La|El|Das)$/$2 $1/og;
+	    $title=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 	    $self->debug("close:$title ($arr[2]) qualifier=$arr[3] id=$arr[4]");
 	    push(@{$results->{closeMatch}}, {'key'=> $arr[1],
 					     'title'=>$title,
@@ -520,11 +520,6 @@ sub getMovieIdDetails($$)
     return($results);
 }
 
-#
-# FUTURE "Columbo Cries Wolf" appears instead of "Columbo:Columbo Cries Wolf"
-#
-# FUTURE - common english->french vowel changes. For instance
-#          "Anna Karnin" (->e)
 #
 # FUTURE - close hit could be just missing or extra
 #          punctuation:
@@ -534,6 +529,11 @@ sub getMovieIdDetails($$)
 #       "Baywatch Hawaiian Wedding" for imdb's "Baywatch: Hawaiian Wedding" :)
 #
 # FIXED - "Victoria and Albert" appears for imdb's "Victoria & Albert" (and -> &)
+# FIXED - "Columbo Cries Wolf" appears instead of "Columbo:Columbo Cries Wolf"
+# FIXED - Place the article last, for multiple languages. For instance
+#         Los amantes del crculo polar -> amantes del crculo polar, Los
+# FIXED - common international vowel changes. For instance
+#          "Anna Karnin" (->e)
 #
 sub alternativeTitles($)
 {
@@ -541,18 +541,59 @@ sub alternativeTitles($)
     my @titles;
 
     push(@titles, $title);
+    # try the & -> and conversion
     if ( $title=~m/\&/o ) {
 	my $t=$title;
 	while ( $t=~s/(\s)\&(\s)/$1and$2/o ) {
 	    push(@titles, $t);
 	}
     }
+    # try the and -> & conversion
     if ( $title=~m/\sand\s/io ) {
 	my $t=$title;
 	while ( $t=~s/(\s)and(\s)/$1\&$2/io ) {
 	    push(@titles, $t);
 	}
     }
+
+    # try the "Columbo: Columbo cries Wolf" -> "Columbo cries Wolf" conversion
+    foreach (@titles) {
+        if ( m/^[^:]+:.+$/io ) {
+	    my $t=$_;
+            while ( $t=~s/^[^:]+:\s*(.+)\s*$/$1/io ) {
+                push(@titles, $t);
+            }
+        }
+    }
+
+    # Place the articles last
+    foreach (@titles) {
+        if ( m/^(The|A|Une|Les|Los|Las|L\'|Le|La|El|Das|De|Het|Een)\s+(.*)$/io ) {
+	    my $t=$_;
+            $t=~s/^(The|A|Une|Les|Los|Las|L\'|Le|La|El|Das|De|Het|Een)\s+(.*)$/$2, $1/iog;
+            push(@titles, $t);
+        }
+    }
+
+    # convert all the special language characters
+    foreach (@titles) {
+	if ( m/[]/io ) {
+	    my $t=$_;
+	    $t=~s/[]/a/gio;
+	    $t=~s/[]/e/gio;
+	    $t=~s/[]/i/gio;
+	    $t=~s/[]/o/gio;
+	    $t=~s/[]/u/gio;
+	    $t=~s/[]/ae/gio;
+	    $t=~s/[]/c/gio;
+	    $t=~s/[]/n/gio;
+	    $t=~s/[]/ss/gio;
+	    $t=~s/[]/y/gio;
+	    $t=~s/[¿]//gio;
+	    push(@titles, $t);
+	}
+    }
+
     return(\@titles);
 }
 
@@ -718,11 +759,11 @@ sub findMovieInfo($$$$)
 		}
 		else {
 		    # report these as debug messages
-		    $self->debug("ignoing close hit on \"$info->{key}\" (off by $yearsOff years)");
+		    $self->debug("ignoring close hit on \"$info->{key}\" (off by $yearsOff years)");
 		}
 	    }
 	    else {
-		$self->debug("ignoing close hit on \"$info->{key}\" (title did not match)");
+		$self->debug("ignoring close hit on \"$info->{key}\" (title did not match)");
 	    }
 	}
     }
@@ -1465,14 +1506,23 @@ sub readMoviesOrGenres($$$$)
 	last if ( $line=~m/^\-\-\-\-\-\-\-+/o );
 
 	$line=~s/\n$//o;
-	
+
 	my $tab=index($line, "\t");
 	if ( $tab != -1 ) {
 	    my $mkey=substr($line, 0, $tab);
 
+	    next if ($mkey=~m/\s*\{\{SUSPENDED\}\}/o);
+
 	    if ( $whatAreWeParsing == 2 ) {
+		# don't see what these are...?
+		# ignore {{SUSPENDED}}
+		$mkey=~s/\s*\{\{SUSPENDED\}\}//o;
+
+		# ignore {Twelve Angry Men (1954)}
+		$mkey=~s/\s*\{[^\}]+\}//go;
+	
 		# skip enties that have {} in them since they're tv episodes
-		next if ( $mkey=~s/\s*\{[^\}]+\}$//o );
+		#next if ( $mkey=~s/\s*\{[^\}]+\}$//o );
 
 		my $genre=substr($line, $tab);
 
@@ -1642,10 +1692,15 @@ sub readCastOrDirectors($$$)
 		# ignore character name
 	    }
 	}
-	if ( $line=~s/\s*\{[^\}]+\}//o ) {
-	    # ignore {Twelve Angry Men (1954)}
-	    # don't see what these are...?
-	}
+	# try ignoring these
+	next if ($line=~m/\s*\{\{SUSPENDED\}\}/o);
+
+	# don't see what these are...?
+	# ignore {{SUSPENDED}}
+	$line=~s/\s*\{\{SUSPENDED\}\}//o;
+
+	# ignore {Twelve Angry Men (1954)}
+	$line=~s/\s*\{[^\}]+\}//o;
 
 	if ( $whatAreWeParsing < 3 ) {
 	    if ( $line=~s/\s*\(aka ([^\)]+)\).*$//o ) {
@@ -1822,6 +1877,40 @@ sub dbinfoSave($)
     return(0);
 }
 
+sub dbinfoCalcEstimate($$$)
+{
+    my ($self, $key, $estimateSizePerEntry)=@_;
+    
+    if ( !defined($self->{imdbListFiles}->{$key}) ) {
+	die ("invalid call");
+    }
+    my $fileSize=int(-s "$self->{imdbListFiles}->{$key}");
+
+    # if compressed, then attempt to run gzip -l
+    if ( $self->{imdbListFiles}->{$key}=~m/.gz$/) {
+	if ( open(my $fd, "gzip -l ".$self->{imdbListFiles}->{$key}."|") ) {
+	    # if parse fails, then defalt to wild ass guess of compression of 65%
+	    $fileSize=int($fileSize*100)/(100-65);
+
+	    while(<$fd>) {
+		if ( m/^\s*\d+\s+(\d+)/ ) {
+		    $fileSize=$1;
+		}
+	    }
+	    close($fd);
+	}
+	else {
+	    # wild ass guess of compression of 65%
+	    $fileSize=int($fileSize*100)/(100-65);
+	}
+    }
+    my $countEstimate=int($fileSize/$estimateSizePerEntry);
+
+    $self->dbinfoAdd($key."_list_file", $self->{imdbListFiles}->{$key});
+    $self->dbinfoAdd($key."_list_file_size", $fileSize);
+    $self->dbinfoAdd($key."_list_count_estimate", $countEstimate);
+    return($countEstimate);
+}
 
 sub invokeStage($$)
 {
@@ -1830,7 +1919,8 @@ sub invokeStage($$)
     my $startTime=time();
     if ( $stage == 1 ) {
 	$self->status("parsing Movies list for stage $stage..");
-	my $countEstimate=398000;
+	my $countEstimate=$self->dbinfoCalcEstimate("movies", 43);
+
 	my $num=$self->readMoviesOrGenres("Movies", $countEstimate, "$self->{imdbListFiles}->{movies}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -1841,8 +1931,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for movies needs updating, found $num");
 	}
-	$self->dbinfoAdd("movie_list_file",         "$self->{imdbListFiles}->{movies}");
-	$self->dbinfoAdd("movie_list_file_size", -s "$self->{imdbListFiles}->{movies}");
 	$self->dbinfoAdd("db_stat_movie_count", "$num");
 
 	$self->status("writing stage1 data ..");
@@ -1881,7 +1969,8 @@ sub invokeStage($$)
     elsif ( $stage == 2 ) {
 	$self->status("parsing Directors list for stage $stage..");
 
-	my $countEstimate=85000;
+	my $countEstimate=$self->dbinfoCalcEstimate("directors", 184);
+
 	my $num=$self->readCastOrDirectors("Directors", $countEstimate, "$self->{imdbListFiles}->{directors}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -1892,8 +1981,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for directors needs updating, found $num");
 	}
-	$self->dbinfoAdd("directors_list_file",         "$self->{imdbListFiles}->{directors}");
-	$self->dbinfoAdd("directors_list_file_size", -s "$self->{imdbListFiles}->{directors}");
 	$self->dbinfoAdd("db_stat_director_count", "$num");
 
 	$self->status("writing stage2 data ..");
@@ -1948,7 +2035,8 @@ sub invokeStage($$)
 	$self->status("parsing Actors list for stage $stage..");
 
 	#print "re-reading movies into memory for reverse lookup..\n";
-	my $countEstimate=508000;
+	my $countEstimate=$self->dbinfoCalcEstimate("actors", 349);
+
 	my $num=$self->readCastOrDirectors("Actors", $countEstimate, "$self->{imdbListFiles}->{actors}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -1959,8 +2047,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for actors needs updating, found $num");
 	}
-	$self->dbinfoAdd("actors_list_file",         "$self->{imdbListFiles}->{actors}");
-	$self->dbinfoAdd("actors_list_file_size", -s "$self->{imdbListFiles}->{actors}");
 	$self->dbinfoAdd("db_stat_actor_count", "$num");
 
 	$self->status("writing stage3 data ..");
@@ -1999,7 +2085,7 @@ sub invokeStage($$)
     elsif ( $stage == 4 ) {
 	$self->status("parsing Actresses list for stage $stage..");
 
-	my $countEstimate=308000;
+	my $countEstimate=$self->dbinfoCalcEstimate("actresses", 311);
 	my $num=$self->readCastOrDirectors("Actresses", $countEstimate, "$self->{imdbListFiles}->{actresses}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -2010,8 +2096,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for actresses needs updating, found $num");
 	}
-	$self->dbinfoAdd("actresses_list_file",         "$self->{imdbListFiles}->{actresses}");
-	$self->dbinfoAdd("actresses_list_file_size", -s "$self->{imdbListFiles}->{actresses}");
 	$self->dbinfoAdd("db_stat_actress_count", "$num");
 
 	$self->status("writing stage4 data ..");
@@ -2049,7 +2133,8 @@ sub invokeStage($$)
     }
     elsif ( $stage == 5 ) {
 	$self->status("parsing Genres list for stage $stage..");
-	my $countEstimate=271000;
+	my $countEstimate=$self->dbinfoCalcEstimate("genres", 61);
+
 	my $num=$self->readMoviesOrGenres("Genres", $countEstimate, "$self->{imdbListFiles}->{genres}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -2060,8 +2145,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for genres needs updating, found $num");
 	}
-	$self->dbinfoAdd("genres_list_file",         "$self->{imdbListFiles}->{genres}");
-	$self->dbinfoAdd("genres_list_file_size", -s "$self->{imdbListFiles}->{genres}");
 	$self->dbinfoAdd("db_stat_genres_count", "$num");
 
 	$self->status("writing stage5 data ..");
@@ -2099,7 +2182,8 @@ sub invokeStage($$)
     }
     elsif ( $stage == 6 ) {
 	$self->status("parsing Ratings list for stage $stage..");
-	my $countEstimate=85000;
+	my $countEstimate=$self->dbinfoCalcEstimate("ratings", 63);
+
 	my $num=$self->readRatings($countEstimate, "$self->{imdbListFiles}->{ratings}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -2110,8 +2194,6 @@ sub invokeStage($$)
 	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
 	    $self->status("ARG estimate of $countEstimate for ratings needs updating, found $num");
 	}
-	$self->dbinfoAdd("ratings_list_file",         "$self->{imdbListFiles}->{ratings}");
-	$self->dbinfoAdd("ratings_list_file_size", -s "$self->{imdbListFiles}->{ratings}");
 	$self->dbinfoAdd("db_stat_ratings_count", "$num");
 
 	$self->status("writing stage6 data ..");
@@ -2395,7 +2477,7 @@ sub invokeStage($$)
 
 		my $val=$movies{$dbkey};
 		if ( !defined($val) ) {
-		    $self->error("genres list references unidentified title '$1'");
+		    $self->error("ratings list references unidentified title '$1'");
 		    next;
 		}
 		$movies{$dbkey}.=$tab.$ratingDist.$tab.$ratingVotes.$tab.$ratingRank;
@@ -2456,6 +2538,9 @@ sub invokeStage($$)
 	    for my $key (keys %movies) {
 		my $dbkey=$key;
 		
+		# drop episode information - ex: {Twelve Angry Men (1954)}
+		$dbkey=~s/\s*\{[^\}]+\}//go;
+		
 		# todo - this would make things easier
 		# change double-quotes around title to be (made-for-tv) suffix instead 
 		if ( $dbkey=~m/^\"/o && #"
@@ -2507,9 +2592,10 @@ sub invokeStage($$)
 		    $year="0000";
 		}
 		else {
-		    die "unable to decode year from title key \"$title\", report to xmltv-devel\@lists.sf.net";
+		    $self->error("movie list format failed to decode year from title '$title'");
+		    $year="0000";
 		}
-		$title=~s/(.*),\s*(The|A|Une|Les|L\'|Le|La|El|Das)$/$2 $1/og;
+		$title=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 		
 		my $hashkey=lc("$title ($year)");
 		$hashkey=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/oeg;
@@ -2694,7 +2780,7 @@ sub crunchStage($$)
 	}
     }
 
-    if ( -f "$self->{moviedbInfo}" ) {
+    if ( -f "$self->{moviedbInfo}" && $stage != 1 ) {
 	my $ret=$self->dbinfoLoad();
 	if ( $ret ) {
 	    $self->error($ret);
@@ -2712,8 +2798,8 @@ sub crunchStage($$)
 	}
 	else {
 	    $self->status("prep stage $stage succeeded with $self->{errorCountInLog} errors in $self->{imdbDir}/stage$stage.log");
-	    if ( $stage == 7 && $self->{errorCountInLog} > 30 && $self->{errorCountInLog} < 60 ) {
-		$self->status("this stage commonly produces around 49 (or so) warnings because of imdb");
+	    if ( $stage == 7 && $self->{errorCountInLog} > 30 && $self->{errorCountInLog} < 80 ) {
+		$self->status("this stage commonly produces around 60 (or so) warnings because of imdb");
 		$self->status("list file inconsistancies, they can usually be safely ignored");
 	    }
 	}
